local mod = ...



-- ************************************************************************************************
-- Component: Controller --------------------------------------------------------------------------
-- ************************************************************************************************

-- Component
local COMP_FIRE_CONTROLLER = {
	TypeName = "COMP_FIRE_CONTROLLER",
	ParentType = "COMPONENT",
	Properties = {
		{ Name = "BuilderWorkshopList", Type = "list<string>", Default = nil, Flags = { "SAVE_GAME" } },
    }
}


function COMP_FIRE_CONTROLLER:init()


    -- set up daily event
    local compMainGameLoop = self:getLevel():find("COMP_MAIN_GAME_LOOP")
    event.register(self, compMainGameLoop.ON_NEW_DAY, function()

        -- Find out current population
        local countPop = 0
        self:getLevel():getComponentManager("COMP_VILLAGER"):getAllEnabledComponent():forEach(function(compVill)
            countPop = countPop + 1
        end)


        -- Chance that a fire will start
        local chance = math.random()
        if (chance < (1/100)*(countPop/100)) and countPop >= 30 then

            -- Record position of all building parts
            local listPos = {}
            self:getLevel():getComponentManager("COMP_BUILDING_PART"):getAllEnabledComponent():forEach(function(compPart)

                local objPart = compPart:getOwner()
                if objPart then
                    local posPart = objPart:getGlobalPosition()
                    table.insert(listPos, posPart)
                end

            end)

            -- Pick a random index from the table
            local posStart = nil
            if #listPos > 0 then
                local i = math.random(1, #listPos)
                posStart = listPos[i]
            end

            -- Start the fire
            if posStart then

                -- Create object
                self:getLevel():createObject("PREFAB_FIRE_ORIGIN", posStart)

                -- Notification
                local compVillager = self:getLevel():getComponentManager("COMP_VILLAGER"):getFirstEnabled()
                local jobAsset = nil
                local jobWorkplace = nil
                if (compVillager) then

                    local job = compVillager:getJobInstance()
                    if (job) then
                        jobAsset = job.AssetJob
                        jobWorkplace = job.Workplace
                    end

                    compVillager:changeJob("JOB_FIRE_EVENT_START")

                    if (job) then
                        if (jobWorkplace) then
                            compVillager:changeJob(jobAsset, jobWorkplace)
                        else
                            compVillager:changeJob(jobAsset)
                        end
                    else
                        compVillager:changeJob(nil)
                    end

                end -- Villager found to trigger the notification

            end -- Position found for starting the fire

        end -- if chance

    end) -- end daily event


end



function COMP_FIRE_CONTROLLER:registerFire(objFire)


    -- Find vanilla builder workshops
    self:findBuilder(objFire, "BuilderWorkshopCorePart")

    -- Find Frontier Essentials mod workshops
    if (foundation.isModLoaded("60041e79-3595-49fc-8ce3-0a3c6d805be2")) then
        self:findBuilder(objFire, "BuilderPart")
    end


end -- registerFire


function COMP_FIRE_CONTROLLER:findBuilder(objFire, objName)


    -- Set variables
    local posFire = objFire:getGlobalPosition()

    -- Make a list of builders workshops within range
    local listBuilderWorkshop = {}
    self:getLevel():find(objName, listBuilderWorkshop)

    if listBuilderWorkshop then
        for i = 1, #listBuilderWorkshop do

            local objWork = listBuilderWorkshop[i]
            local posWork = objWork:getGlobalPosition()
            local compPart = objWork:getComponent("COMP_BUILDING_PART")
            local dist = math.sqrt( (posFire[1] - posWork[1])^2 + (posFire[3] - posWork[3])^2 )

            if (dist <= 100) and objWork.Active and compPart:getIsBuilt() then

                -- Record the workshop for later
                table.insert(self.BuilderWorkshopList, {objFire = objFire:getId(), objBuilder = objWork:getId()})

                -- Change the workplace function to fighting fires
                local compWork = objWork:getComponent("COMP_WORKPLACE")
                if compWork then

                    -- change jobs
                    local listWorker = compWork.AssignedWorkerList
                    if listWorker then
                        for n = 1, #listWorker do
                            listWorker[n]:changeJob("JOB_FIRE_WATER")
                        end
                    end

                    -- change function
                    compWork:setWorkplaceData("BUILDING_FUNCTION_FIRE_WATER")

                end -- workplace exists

            end -- within range, active and built

        end -- loop through builder objects

    end -- there is at least one builder workshop

end



function COMP_FIRE_CONTROLLER:unregisterFire(objFire)

    -- Remove records for the current fire
    local removeList = {}

    if #self.BuilderWorkshopList > 0 then

        local idFire = objFire:getId()
        for i = #self.BuilderWorkshopList, 1, -1 do

            local rec = self.BuilderWorkshopList[i]
            if rec.objFire == idFire then
                table.insert(removeList, self.BuilderWorkshopList[i].objBuilder)
                table.remove(self.BuilderWorkshopList, i)
            end

        end

    end


    -- If this is the last record of that builder workshop, turn it back to a builders
    if #removeList > 0 then

        -- Loop through all removed workshops
        for r = 1, #removeList do

            local found = false
            if #self.BuilderWorkshopList > 0 then

                for b = #self.BuilderWorkshopList, 1, -1 do

                    local rec = self.BuilderWorkshopList[b]
                    if rec.objBuilder == removeList[r] then
                        found = true
                    end
                end

            end

            if not found then

                local objBuilder = self:getLevel():find(removeList[r])
                if objBuilder then

                    local compWork = objBuilder:getComponent("COMP_WORKPLACE")
                    if compWork then

                        -- change jobs
                        local listWorker = compWork.AssignedWorkerList
                        if listWorker then
                            for n = 1, #listWorker do
                                listWorker[n]:changeJob("BUILDER")
                            end
                        end

                        -- change function
                        compWork:setWorkplaceData("BUILDING_FUNCTION_BUILDER_WORKSHOP")

                    end -- workplace exists

                end -- builder exists

            end -- builder is no longer tied to any fire

        end -- loop through remove list

    end -- remove list is populated

end -- unregister fire


-- Register
mod:registerClass(COMP_FIRE_CONTROLLER)



-- Attach component
mod:registerPrefabComponent("PREFAB_MANAGER", { DataType = "COMP_FIRE_CONTROLLER" } )


